#ifndef COMPGEOM_H
#define COMPGEOM_H

#include "point.h"

class CompGeom
{
 public:
  enum IntersectionType{ DO_NOT_INTERSECT, DO_INTERSECT,
                         COLLINEAR, PARALLEL, TOUCH };

  // This function returns a flag indicating whether a point 'p' is
  // close to a line segment 'p1'-'p2'. Two tolerance values are given
  // in x and y direction in variable '_tol'.
  static bool isPointCloseToSegment( Point _p1, Point _p2, Point _p, Point _tol );

  // Get closest point on line 'p1'-'p2'.
  // Returns the distance between given point and closest point.
  // Also returns parametric value (from -infinity to +infinity)
  // of closest point along the line.
  static double getClosestPointLine( Point _p1, Point _p2, Point _p,
                                     Point& _pC, double& _t );

  // Get closest point on line segment 'p1'-'p2'.
  // Returns the distance between given point and closest point.
  // Also returns parametric value (between 0 and 1) of closest point
  // along the line.
  // The difference between this function and function 'getClosestPointLine'
  // is that a closest point outside the limits of segment 'p1-p2' is 
  // snapped to one of the segment end points.
  static double getClosestPointSegment( Point _p1, Point _p2, Point _p,
                                        Point& _pC, double& _t );

  // Check for collinear segments 'p1'-'p2' and 'p3'-'p4'.
  static bool checkCollinearSegments( Point _p1, Point _p2,
                                      Point _p3, Point _p4 );

  // Checks for two line segments intersection:
  // Checks whether segment 'p1'-'p2' intercepts segment 'p3'-'p4'.
  // Returns an integer intersection type value
  // In case there is intersection, outputs the result in 'pi' parameter
  // and returns parameters values (between 0 and 1) in along the two segments.
  // Ref.: John Vince - Geometry for Computer Graphics - Springer 2005
  static int computeSegmentSegmentIntersection( Point _p1, Point _p2,
                                                Point _p3, Point _p4,
                                                Point& _pi,
                                                double& _t12, double& _t34 );

  // This function returns a flag indicating whether the vertices
  // of a polygon are in counter-clockwise order.
  // The algorithm is as follows:
  // Traverse the loop of coordinates, assuming that it is in counter-
  // clockwise order, computing the components of the "area" of the
  // enclosed polygon.  The total "area" components are computed by
  // adding "area" components (cross product components) of
  // triangles sides formed by the first, previous, and current
  // vertices.  If the loop is not convex, some of the triangle
  // areas might be negative, but those will be compensated by other
  // positive triangle areas so that the final area is positive.
  // (area here is actually twice the area).
  // (positive here means in the direction of the face normal).
  static bool isCounterClockwisePolygon( vector<Point>& _poly );

  // This function returns a flag indicating whether the given point 'p'
  // is inside a polygon 'poly'.
  // The algorithm counts the number of intersections that a horizontal
  // line emanating at the point 'p' in positive x direction makes with
  // the boundary lines of the polygon. If the number of intersections
  // is odd, the point is inside de polygon. Otherwise, it is outside
  // the polygon.
  // If ray passes at an horizontal segment on polygon boundary, do not
  // count any intersection.
  // If ray passes at a vertex of the polygon, only consider intersection
  // if boundary segment is above ray.
  static bool isPointInPolygon( vector<Point>& _poly, Point _p );

 private:
  static double ABSTOL;       // Absolute tolerance value.

  // Assemble four int vector code for line pick
  static void pickLineCode( double x, double y,
                            double xmin, double xmax, double ymin, double ymax,
                            int cod[4] );

};

#endif // COMPGEOM_H
